/* SPDX-License-Identifier: LGPL-3.0-or-later */

/*
 * Copyright (C) 2021 Perry Werneck <perry.werneck@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

 #include <config.h>
 #include <udjat/civetweb.h>
 #include <udjat/tools/http/request.h>
 #include <iostream>
 #include <sstream>

 using namespace std;

 namespace Udjat {

	HTTP::Response::Response(Udjat::MimeType type) {
		this->type = type;
		this->value = new HTTP::Value();
	}

	HTTP::Response::~Response() {
		delete this->value;
	}

	bool HTTP::Response::isNull() const {
		return this->value->isNull();
	}

	std::string HTTP::Response::to_string() const {

		std::stringstream ss;

		switch(type) {
		case Udjat::MimeType::xml:
			// Format as XML
			ss << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
			ss << "<response>";
			this->value->xml(ss);
			ss << "</response>";
			break;

		case Udjat::MimeType::json:
			// Format as JSON
			this->value->json(ss);
			break;

		case Udjat::MimeType::html:
			// Format as HTML.
			this->value->html(ss);
			break;

		default:
			throw runtime_error("Invalid mymetype");

		}

		return ss.str();
	}

	Udjat::Value & HTTP::Response::operator[](const char *name) {
		return (*this->value)[name];
	}

	Udjat::Value & HTTP::Response::append(const Type type) {
		return this->value->append(type);
	}

	Udjat::Value & HTTP::Response::reset(const Type type) {
		return this->value->reset(type);
	}

	Udjat::Value & HTTP::Response::set(const Value &value) {
		return this->value->set(value);
	}

	Udjat::Value & HTTP::Response::set(const char *value, const Type type) {
		return this->value->set(value,type);
	}

 }
